using System.IO;
using UnityEngine;

namespace Magify
{
    internal class ContextFileStorage : FileStorage, ILegacyMigrator
    {
        private const string ConfigJson = "config.json";
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Storage);

        public ContextFileStorage(string storagePath) : base(storagePath)
        {
        }

        public ContextFileStorage(string rootFolderName, string storagePath) : base(rootFolderName, storagePath)
        {
        }

        void ILegacyMigrator.Migrate(MigrationData data)
        {
            _logger.Log($"Try to migrate context file from old path: {data.ContextPath}");
            if (Exists(ConfigJson))
            {
                _logger.Log($"Saved context file already exists at path {ConfigJson}. Skip migration.");
                return;
            }
            if (data.ContextPath == null || !File.Exists(data.ContextPath))
            {
                _logger.Log($"Old context file doesn't exists at path {data.ContextPath}. Skip migration.");
                return;
            }
            _logger.Log($"Old context file found at path {data.ContextPath}. Start migration to new file {ConfigJson}");
            CopyTextFrom(data.ContextPath, ConfigJson);
            _logger.Log($"Old context file successfully migrated to new path {ConfigJson}.");
        }

        public void SaveContext(string json)
        {
            Save(ConfigJson, json);
        }

        public string LoadContext()
        {
            return Load(ConfigJson);
        }

        public void ResetContext()
        {
            Clear(ConfigJson);
        }
    }
}